from numpy import *
from matplotlib.pyplot import *

radius = 130e-3 / 2
sources_per_axis = 100

src_positions = empty([0, 3])
src_amplitudes = empty(0) 

for x in linspace(-radius, radius, sources_per_axis):
    for y in linspace(-radius, radius, sources_per_axis):
        if linalg.norm([x, y]) <= radius:
            src_positions = vstack([src_positions, [x, y, 0]])
            src_amplitudes = append(src_amplitudes, [1 + 0j])

def amplitude_at(pos, wavelength):
    distances = linalg.norm(pos - src_positions, axis=1)
    
    # -2 pi radians per wavelength
    phase_factor = exp(-2j * pi * distances / wavelength)

    # amplitude inversely proportional to distance (spherical wave)
    amplitude_factor = 1 / distances

    return sum(src_amplitudes * phase_factor * amplitude_factor)

def plot_directivity(distance=1, wavelength=0.1):
    phis = linspace(-pi/2, pi/2, 301)
    A = array([amplitude_at(distance * array([sin(phi), 0, cos(phi)]), wavelength) for phi in phis])
    on_axis = A[len(A) // 2]
    A /= on_axis

    label=f'wl={wavelength:.2f} m; d={distance:.1f} m'
    figure(1)
    bodeplot(phis*180/pi, A, label=label)
    xlabel('Angle off-axis [degrees]')

    # see https://stackoverflow.com/questions/26137222/a-half-polar-plot-directivity-pattern-with-db-scale-and-negative-angle-values
    figure(2)
    ax = subplot(111, polar=True)
    ax.set_theta_zero_location('E')
    #ax.set_theta_direction('clockwise')
    plot(phis, 20*log10(abs(A)), label=label)
    ax.set_ylim(-20,0)
    ax.set_yticks(np.array([-30, -20, -12, -6, -3, 0]))
    #ax.set_xticks(np.array([0, -45, -90, np.nan, np.nan, np.nan, 90, 45])/180*pi)
    legend(loc='lower left')

def bodeplot(x, A, label):
    subplot(2,1,1)
    plot(x, 20*log10(abs(A)), label=label)
    ylabel('Amplitude [dB]')

    subplot(2,1,2)
    plot(x, angle(A)*180/pi, label=label)
    ylabel('Phase angle [degrees]')
    legend(loc='lower left')

figure(1)
clf()
figure(2)
clf()

[plot_directivity(distance=1, wavelength=344/f) for f in [1e3, 2e3, 3e3, 4e3]]

show()

